﻿using System;
using System.Security.Cryptography;
using System.Text;

namespace TestProject
{
    /// <summary>
    /// 해시 헬퍼
    /// </summary>
    public class HashHelper
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Public

        #region 솔트 생성하기 - CreateSalt(size)

        /// <summary>
        /// 솔트 생성하기
        /// </summary>
        /// <param name="size">크기</param>
        /// <returns>솔트</returns>
        public static string CreateSalt(int size)
        {
            RNGCryptoServiceProvider provider = new RNGCryptoServiceProvider();

            byte[] targetByteArray = new byte[size];

            provider.GetBytes(targetByteArray);

            return Convert.ToBase64String(targetByteArray);
        }

        #endregion
        #region 해시 생성하기 - CreateHash(source, salt)

        /// <summary>
        /// 해시 생성하기
        /// </summary>
        /// <param name="source">소스 문자열</param>
        /// <param name="salt">솔트</param>
        /// <returns>해시</returns>
        public static string CreateHash(string source, string salt)
        {
            byte[] sourceByteArray = Encoding.UTF8.GetBytes(source + salt);

            SHA256Managed sha256Managed = new SHA256Managed();

            byte[] hashByteArray = sha256Managed.ComputeHash(sourceByteArray);

            return Convert.ToBase64String(hashByteArray);
        }

        #endregion
        #region 해시 검증하기 - VerifyHash(source, hash, salt)

        /// <summary>
        /// 해시 검증하기
        /// </summary>
        /// <param name="source">소스 문자열</param>
        /// <param name="hash">해시</param>
        /// <param name="salt">솔트</param>
        /// <returns>해시 검증 결과</returns>
        public static bool VerifyHash(string source, string hash, string salt)
        {
            string newHash = CreateHash(source, salt);

            return newHash.Equals(hash);
        }

        #endregion
     }
}